/*******************************************************************************
 *******************************************************************************
 *
 * BEGIN WINDJAY GPL
 *
 * Copyright (c) 2007 Erik Staats.
 * http://www.windjay.com
 *
 * This file may be licensed under the terms of of the
 * GNU General Public License Version 2 (the "GPL").
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either 
 * express or implied. See the GPL for the specific language 
 * governing rights and limitations.
 *
 * You should have received a copy of the GPL along with this 
 * program. If not, go to http://www.gnu.org/licenses/gpl.html
 * or write to the Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * END WINDJAY GPL
 *
 *******************************************************************************
 *
 * Javascript source file for Hype Machine site.
 *
 *******************************************************************************
 ******************************************************************************/

/*******************************************************************************
 *******************************************************************************
 *
 * Hype machine config.
 *
 *******************************************************************************
 ******************************************************************************/

var HMConfig =
{
    /*
     * siteDomain                   Target site domain.
     * searchMenuItemID             Search menu item ID.
     * searchMenuItemLabel          Search menu item label.
     * searchMenuItemImage          Search menu item image URL.
     */

    siteDomain: "hypem.com",
    searchMenuItemID: "search.popup.hypem",
    searchMenuItemLabel: "Hype Machine",
    searchMenuItemImage: "http://www.hypem.com/favicon.ico"
}


/*******************************************************************************
 *******************************************************************************
 *
 * Hype Machine site overlay services.
 *
 *******************************************************************************
 ******************************************************************************/

var HMOverlay =
{
    /***************************************************************************
     *
     * Site overlay configuration.
     *
     **************************************************************************/

    /*
     * siteDomain                   Target site domain.
     * searchMenuItemID             Search menu item ID.
     * searchMenuItemLabel          Search menu item label.
     * searchMenuItemImage          Search menu item image URL.
     */

    siteDomain: HMConfig.siteDomain,
    searchMenuItemID: HMConfig.searchMenuItemID,
    searchMenuItemLabel: HMConfig.searchMenuItemLabel,
    searchMenuItemImage: HMConfig.searchMenuItemImage,


    /***************************************************************************
     *
     * Site overlay fields.
     *
     **************************************************************************/

    /*
     * mInitializeFunc              Site initialization function.
     * mHandleMainWindowUnloadFunc  Main Songbird window load handler.
     * mHandleMainBrowserLoadFunc   Main Songbird browser load handler.
     * mSBMainWindow                Main Songbird window.
     * mSBMainBrowser               Main Songbird browser.
     */

    mInitializeFunc: null,
    mHandleMainBrowserLoadFunc: null,
    mHandleMainWindowUnloadFunc: null,
    mSBMainWindow: null,
    mSBMainBrowser: null,


    /***************************************************************************
     *
     * Site overlay functions.
     *
     **************************************************************************/

    /*
     * initialize
     *
     *   This function initializes the site overlay services.
     *
     *   The initialization code is run after the main Songbird window DOM has
     * been loaded, so that all the Songbird main window XUL elements are
     * usable.
     */

    initialize: function()
    {
        var                         sbMainWindow;

        /* Get the main Songbird window. */
        sbMainWindow = document.getElementById("frame_outer");

        /* Create an initialize function with an object closure. */
        {
            var                         _this = this;

            this.mInitializeFunc = function()
            {
                _this.initialize1();
            }
        }

        /* Run initialization code after the main */
        /* Songbird window DOM has been loaded.   */
        /*zzz how can this be undone when window closed? */
        /*zzz unload is called when window is reloaded. */
        sbMainWindow.addEventListener("DOMContentLoaded",
                                      this.mInitializeFunc,
                                      true);
    },

    initialize1: function()
    {
        var                         mainBrowser;

        /* Ensure site overlay services are inactive. */
        this.deactivate();

        /* Activate the site overlay services. */
        this.activate();
    },


    /*
     * activate
     *
     *   This function activates the site overlay services.
     */

    activate: function()
    {
        /* Get the main Songbird window and browser. */
        this.mSBMainWindow = document.getElementById("frame_outer");
        this.mSBMainBrowser = document.getElementById("frame_main_pane");

        /* Create handleMainWindowUnload and handleMainBrowserLoad */
        /* functions with an object closure.                       */
        {
            var                         _this = this;

            this.mHandleMainWindowUnloadFunc = function(event)
            {
                _this.handleMainWindowUnload(event);
            }

            this.mHandleMainBrowserLoadFunc = function(event)
            {
                _this.handleMainBrowserLoad(event);
            }
        }

        /* Register to receive unload events for the main Songbird window. */
        this.mSBMainWindow.addEventListener("unload",
                                            this.mHandleMainWindowUnloadFunc,
                                            true);

        /* Register to receive load events for the main Songbird browser. */
        this.mSBMainBrowser.addEventListener("DOMContentLoaded",
                                             this.mHandleMainBrowserLoadFunc,
                                             true);

        /* Activate the site overlay search services. */
        this.searchActivate();
    },


    /*
     * deactivate
     *
     *   This function deactivates the site overlay services.
     */

    deactivate: function()
    {
        /* Deactivate the site overlay search services. */
        this.searchDeactivate();

        /* Remove main Songbird window listeners and references. */
        if (this.mSBMainWindow)
        {
            this.mSBMainWindow.removeEventListener
                                            ("unload",
                                             this.mHandleMainWindowUnloadFunc,
                                             true);
            this.mHandleMainWindowUnloadFunc = false;
            this.mSBMainWindow = null;
        }

        /* Remove main Songbird browser listeners and references. */
        if (this.mSBMainBrowser)
        {
            this.mSBMainBrowser.removeEventListener
                                            ("DOMContentLoaded",
                                             this.mHandleMainBrowserLoadFunc,
                                             true);
            this.mHandleMainBrowserLoadFunc = false;
            this.mSBMainBrowser = null;
        }
    },


    /*
     * isTargetSite
     *
     *   --> uri                    Host URI to check.
     *
     *   <-- true                   Host URI is from the target site.
     *       false                  Host URI is not from the target site.
     *
     *   This function checks if the host URI specified by uri is from the
     * target site.  If it is, this function returns true; otherwise, it returns
     * false.
     */

    isTargetSite: function(uri)
    {
        var                         hostArray = [];
        var                         targetSiteArray;
        var                         i;
        var                         isTargetSite = true;

        /* Split host and target site domain name     */
        /* strings into domain name component arrays. */
        try { hostArray = uri.host.split("."); } catch (e) {}
        targetSiteArray = this.siteDomain.split(".");

        /* Take a slice from the end of the host name to    */
        /* make it the same length as the target site name. */
        if (hostArray.length >= targetSiteArray.length)
            hostArray = hostArray.slice(-targetSiteArray.length);
        else
            isTargetSite = false;

        /* Compare the host and target site domain names. */
        for (i = 0; (i < targetSiteArray.length) && isTargetSite; i++)
        {
            if (hostArray[i] != targetSiteArray[i])
                isTargetSite = false;
        }

        return (isTargetSite);
    },


    /***************************************************************************
     *
     * Site overlay search services.
     *
     **************************************************************************/
    
    /*
     * mSearchActive                True if the search services have been
     *                              activated.
     * mOnSearchTermFunc            Site onSearchTerm function.
     * mOrigOnSearchTermFunc        Original onSearchTerm function.
     */

    mSearchActive: false,
    mOnSearchTermFunc: null,
    mOrigOnSearchTermFunc: null,


    /*
     * searchActivate
     *
     *   This function activates the site overlay search services.
     */

    searchActivate: function()
    {
        /* Do nothing if already active. */
        if (this.mSearchActive)
            return;

        /* Mark the services as active. */
        this.mSearchActive = true;

        /* Add a site search menu item to each search element. */
        this.addDocumentSearchMenuItem(this.mSBMainWindow);

        /* Add a site search menu item to each playlist element. */
        this.addDocumentPlaylistSearchMenuItem(this.mSBMainWindow);

        /* Create an onSearchTerm function with an object closure. */
        {
            var                         _this = this;

            this.mOnSearchTermFunc = function(target, searchTerm)
            {
                _this.onSearchTerm(target, searchTerm);
            }
        }

        /* Replace the onSearchTerm function with the site version. */
        this.mOrigOnSearchTermFunc = onSearchTerm;
        onSearchTerm = this.mOnSearchTermFunc;
    },


    /*
     * searchDeactivate
     *
     *   This function deactivates the site overlay search services.
     */

    searchDeactivate: function()
    {
    },


    /*
     * addDocumentSearchMenuItem
     *
     *   --> _document              Document to which to add site search menu
     *                              items.
     *
     *   For each search element within the document specified by _document,
     * this function adds a site search menu item.
     */

    addDocumentSearchMenuItem: function(_document)
    {
        var                         searchElementList;
        var                         i;

        /* Add a site search menu item to each search element. */
        searchElementList = _document.getElementsByTagName("search");
        for (i = 0; i < searchElementList.length; i++)
            this.addSearchMenuItem(searchElementList[i]);
    },


    /*
     * addSearchMenuItem
     *
     *   --> searchElement          Search element to which to add site search
     *                              menu item.
     *
     *   This function adds a site menu item to the search menu of the search
     * element specified by searchElement.
     */

    addSearchMenuItem: function(searchElement)
    {
        var                         _document;
        var                         popupElement;
        var                         libraryElement;
        var                         siteElement;
        var                         defaultSearchID;

        /* Get the search document. */
        _document = searchElement.ownerDocument;

        /* Get the search widget elements. */
        popupElement = _document.getAnonymousElementByAttribute
                                                            (searchElement,
                                                             "id",
                                                             "search.popup");
        libraryElement = _document.getAnonymousElementByAttribute
                                                    (searchElement,
                                                     "id",
                                                     "search.popup.songbird");

        /* Create a site search element. */
        siteElement = _document.createElement("menuitem");
        siteElement.setAttribute("id", this.searchMenuItemID);
        siteElement.setAttribute("label", this.searchMenuItemLabel);
        siteElement.setAttribute("class", "menuitem-iconic");
        siteElement.setAttribute("image", this.searchMenuItemImage);
        siteElement.setAttribute("get_search_url",
                                 "return (HMSite.getSearchURL(searchTerm));");

        /* Add the site search element after the Songbird library element. */
        popupElement.insertBefore(siteElement, libraryElement.nextSibling);

        /* Set the site search element to be the default. */
        defaultSearchID = SBDataGetStringValue("search.defaultweb");
        if (   (!defaultSearchID)
            || (defaultSearchID == siteElement.getAttribute("id")))
        {
            searchElement.theLastWebSearchSelection = siteElement;
        }
    },


    /*
     * addDocumentPlaylistSearchMenuItem
     *
     *   --> _document              Document to which to add playlist site
     *                              search menu items.
     *
     *   For each playlist element within the document specified by _document,
     * this function adds a site search menu item.
     */

    addDocumentPlaylistSearchMenuItem: function(_document)
    {
        var                         playlistElementList;
        var                         i;

        /* Add a site search menu item to each playlist element. */
        playlistElementList = _document.getElementsByTagName("playlist");
        for (i = 0; i < playlistElementList.length; i++)
            this.addPlaylistSearchMenuItem(playlistElementList[i]);
    },


    /*
     * addPlaylistSearchMenuItem
     *
     *   --> playlistElement        Playlist element to which to add site search
     *                              menu item.
     *
     *   This function adds a site menu item to the search menu of the playlist
     * element specified by playlistElement.
     */

    addPlaylistSearchMenuItem: function(playlistElement)
    {
        var                         _document;
        var                         popupElement;
        var                         separatorElement;
        var                         firstSearchElement;
        var                         siteElement;

        /* Get the playlist document. */
        _document = playlistElement.ownerDocument;

        /* Get the playlist search menu popup element. */
        popupElement = _document.getAnonymousElementByAttribute
                                                    (playlistElement,
                                                     "id",
                                                     "playlist.popup.search");

        /* Find the first search menu item. */
        separatorElement =
                        popupElement.getElementsByTagName("menuseparator")[0];
        firstSearchElement = separatorElement.nextSibling;

        /* Create a site search element. */
        siteElement = _document.createElement("menuitem");
        siteElement.setAttribute("id", this.searchMenuItemID);
        siteElement.setAttribute("label", this.searchMenuItemLabel);
        siteElement.setAttribute("class", "menuitem-iconic");
        siteElement.setAttribute("image", this.searchMenuItemImage);
        siteElement.setAttribute("get_search_url",
                                 "return (HMSite.getSearchURL(searchTerm));");

        /* Add the site search element as the first search menu item. */
        popupElement.insertBefore(siteElement, firstSearchElement);
    },


    /*
     * onSearchTerm
     *
     *   --> target                 Search target element.
     *   --> searchTerm             Search term.
     *
     *   This function overlays the Songbird onSearchTerm function.  It searches
     * for the search term specified by searchTerm with the site element
     * specified by target.
     */

    onSearchTerm: function(target, searchTerm)
    {
        var                         _document;
        var                         serviceTree;
        var                         getSearchURLFunc;
        var                         searchURL;
        var                         handleSearch = true;

        /* Don't handle search if parameters invalid. */
        if (!target || !searchTerm)
            handleSearch = false;

        /* If the target search element does not provide a            */
        /* "get_search_url" function, call the original onSearchTerm. */
        if (handleSearch)
        {
            if (!target.getAttribute("get_search_url"))
                handleSearch = false;
        }

        /* If not handling search, call the original onSearchTerm. */
        if (!handleSearch)
        {
            this.mOrigOnSearchTermFunc(target, searchTerm);
            return;
        }

        /* Get the search URL. */
        getSearchURLFunc = new Function("searchTerm",
                                        target.getAttribute("get_search_url"));
        searchURL = getSearchURLFunc(searchTerm);

        /* Launch the search URL. */
        serviceTree = document.getElementById("frame_servicetree");
        serviceTree.launchURL(searchURL);
    },


    /***************************************************************************
     *
     * Site overlay event handlers.
     *
     **************************************************************************/

    /*
     * handleMainWindowUnload
     *
     *   --> event                  Browser load event.
     *
     *   This function handles unloading of the main Songbird window as
     * specified by event.
     */

    handleMainWindowUnload: function(event)
    {
        var                         mainWindow;
        var                         mainBrowser;

        /* Deactivate the site overlay services. */
        this.deactivate();
    },


    /*
     * handleMainBrowserLoad
     *
     *   --> event                  Browser load event.
     *
     *   This function handles loading of the main Songbird browser as specified
     * by event.  If a page from the site domain is being loaded in the main
     * Songbird browser, this function will redirect the page through the site
     * XUL.
     */

    handleMainBrowserLoad: function(event)
    {
        try { this.handleMainBrowserLoad1(event); }
        catch (e) { dump("handleMainBrowserLoad exception: " + e + "\n"); }
    },

    handleMainBrowserLoad1: function(event)
    {
        var                         mainBrowser;
        var                         uri;
        var                         isTargetSite = false;

        /* Add a site search menu items to each playlist element. */
        this.addDocumentPlaylistSearchMenuItem
                                        (this.mSBMainBrowser.contentDocument);

        /* Get the main Songbird browser current URI. */
        uri = this.mSBMainBrowser.currentURI;

        /* Check if this is the target site. */
        isTargetSite = this.isTargetSite(uri);

        /* If the target site was loaded, process the loaded page. */
        /* Otherwise, ensure the site services are inactive.       */
        if (isTargetSite)
        {
            HMSite.activate();
            HMSite.processDocument(this.mSBMainBrowser.contentDocument);
        }
        else
        {
            HMSite.deactivate();
        }
    }
}


/*******************************************************************************
 *******************************************************************************
 *
 * Hype Machine site services.
 *
 *******************************************************************************
 ******************************************************************************/

var HMSite =
{
    /***************************************************************************
     *
     * Site configuration.
     *
     **************************************************************************/

    /*
     * siteDomain                   Target site domain.
     * siteDBGUID                   Site database GUID.
     * siteDBTable                  Site playlist database table.
     * sitePlaylistMIMEType         MIME type of site playlists.
     */

    siteDomain: HMConfig.siteDomain,
    siteDBGUID: WEB_PLAYLIST_CONTEXT,
    siteDBTable: WEB_PLAYLIST_TABLE,
    siteLibraryReadableName: WEB_PLAYLIST_LIBRARY_NAME,
    sitePlaylistReadableName: WEB_PLAYLIST_TABLE_NAME,
    sitePlaylistMIMEType: "application/rss+xml",


    /***************************************************************************
     *
     * Site fields.
     *
     **************************************************************************/

    /*
     * mIOService                   I/O service object.
     * mPlaylistManager             Playlist manager object.
     * mDBQuery                     Site media database query object.
     * mPlaylistReaderManager       Playlist reader manager object.
     * mWebBrowser                  Web browser object.
     * mPlaylistFile                Currently downloading playlist file.
     * mSiteTmpDir                  Directory in which to store temporary site
     *                              files.
     * mSiteDocumentURI             URI of displayed site document.
     * mPlaylistURL                 URL object of site playlist file.
     * mHandlePlayClickFunc         Site handlePlayClick function with object
     *                              closure.
     * mActive                      If true, the site services are active.
     */

    mIOService: null,
    mPlaylistManager: null,
    mDBQuery: null,
    mPlaylistReaderManager: null,
    mWebBrowser: null,
    mPlaylistFile: null,
    mSiteTmpDir: null,
    mSiteDocumentURI: null,
    mPlaylistURL: null,
    mHandlePlayClickFunc: null,
    mActive: false,


    /***************************************************************************
     *
     * Site functions.
     *
     **************************************************************************/

    /*
     * activate
     *
     *   This function activates the site services.
     */

    activate: function()
    {
        /* Do nothing if already active. */
        if (this.mActive)
            return;

        /* Activate.  Deactivate on exception. */
        try
        {
            this.activate1();
            this.mActive = true;
        }
        catch (e)
        {
            this.deactivate();
        }
    },
    activate1: function()
    {
        /* Create I/O service, playlist manager, */
        /* and playlist reader manager objects.  */
        this.mIOService = Components.
                                classes["@mozilla.org/network/io-service;1"].
                                getService(Components.interfaces.nsIIOService);
        this.mPlaylistManager = Components.
                    classes["@songbirdnest.com/Songbird/PlaylistManager;1"].
                    createInstance(Components.interfaces.sbIPlaylistManager);
        this.mPlaylistReaderManager = Components.
                classes["@songbirdnest.com/Songbird/PlaylistReaderManager;1"].
                createInstance(Components.interfaces.sbIPlaylistReaderManager);

        /* Create and set up a database query object. */
        this.mDBQuery = Components.
                        classes["@songbirdnest.com/Songbird/DatabaseQuery;1"].
                        createInstance(Components.interfaces.sbIDatabaseQuery);
        this.mDBQuery.setAsyncQuery(false);
        this.mDBQuery.setDatabaseGUID(this.siteDBGUID);

        /* Create a directory for temporary site files. */
        this.createTmpDir();

        /* Create a handlePlayClick function with an object closure. */
        {
            var                         _this = this;

            this.mHandlePlayClickFunc = function(event)
            {
                _this.handlePlayClick(event);
            }
        }

        /* Disable the Songbird web playlist services. */
        SBDataSetBoolValue("webplaylist.enabled", false);
    },


    /*
     * deactivate
     *
     *   This function deactivates the site services.
     */

    deactivate: function()
    {
        /* Do nothing if not active. */
        if (!this.mActive)
            return;

        /* No longer active. */
        this.mActive = false;

        /* Re-enable the Songbird web playlist services. */
        SBDataSetBoolValue("webplaylist.enabled", true);

        /* Delete the site temporary directory. */
        this.deleteTmpDir();

        /* Clear the various objects. */
        this.mIOService = null;
        this.mPlaylistManager = null;
        this.mDBQuery = null;
        this.mPlaylistReaderManager = null;
        this.mWebBrowser = null;
    },


    /*
     * processDocument
     *
     *   --> siteDocument           Site document to process.
     *
     *   This function processes the site document specified by siteDocument.
     */

    processDocument: function(siteDocument)
    {
        var                         anchorList;
        var                         anchor;
        var                         linkElementList;
        var                         link;
        var                         i;

        /* Get the site document URI. */
        this.mSiteDocumentURI = SBDataGetStringValue("browser.uri");

        /* Create a playlist URL object. */
        this.mPlaylistURL =
                    Components.
                        classes["@mozilla.org/network/standard-url;1"].
                        createInstance(Components.interfaces.nsIStandardURL);

        /* Add event listeners for all anchors. */
        anchorList = siteDocument.getElementsByTagName("a");
        for (i = 0; i < anchorList.length; i++)
        {
            anchor = anchorList[i];
            anchor.addEventListener("contextmenu", onLinkContext, true);
            anchor.addEventListener("mouseover", onLinkOver, true);
            anchor.addEventListener("mouseout", onLinkOut, true);
        }

        /* Process the song info. */
        this.processDocumentSongInfo(siteDocument);

        /* Find the playlist URL on the site document. */
        linkElementList = siteDocument.getElementsByTagName("link");
        for (i = 0; i < linkElementList.length; i++)
        {
            link = linkElementList[i];
            if (   (link.getAttribute("rel") == "alternate")
                && (link.getAttribute("title") == "RSS"))
            {
                this.mPlaylistURL.init(this.mPlaylistURL.URLTYPE_STANDARD,
                                       0,
                                       link.getAttribute("href"),
                                       null,
                                       null);
                break;
            }
        }

        /* Recast mPlaylistURL to nsIURL */
        this.mPlaylistURL =
                this.mPlaylistURL.QueryInterface(Components.interfaces.nsIURL);

        /* Create a temporary local playlist file. */
        this.mPlaylistFile = this.mSiteTmpDir.clone();
        this.mPlaylistFile.append(this.mPlaylistURL.fileName);
        this.mPlaylistFile.createUnique
                                (Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
                                 this.mSiteTmpDir.permissions);

        /* Create an set up the web browser object. */
        this.mWebBrowser =
            Components.
                classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].
                createInstance(Components.interfaces.nsIWebBrowserPersist);
        this.mWebBrowser.progressListener = this;
        this.mWebBrowser.persistFlags =
                        this.mWebBrowser.PERSIST_FLAGS_REPLACE_EXISTING_FILES;

        /* Download the playlist. */
        this.mWebBrowser.saveURI(this.mPlaylistURL,
                                 null,
                                 null,
                                 null,
                                 null,
                                 this.mPlaylistFile);
    },


    /*
     * processDocumentSongInfo
     *
     *   --> siteDocument           Site document to process.
     *
     *   This function processes the song info table data in the site document
     * specified by siteDocument.
     */

    processDocumentSongInfo: function(siteDocument)
    {
        var                         tableDataList;
        var                         tableData;
        var                         songInfoList = [];
        var                         songInfo;
        var                         songInfoAnchorList;
        var                         songInfoAnchor;
        var                         handlePlayClickFunc;
        var                         i, j;

        /* Produce the list of song info table data. */
        tableDataList = siteDocument.getElementsByTagName("td");
        for (i = 0; i < tableDataList.length; i++)
        {
            tableData = tableDataList[i];
            if (tableData.className == "songinfo_cell")
                songInfoList.push(tableData);
        }

        /* Extract info from the song info list. */
        for (i = 0; i < songInfoList.length; i++)
        {
            /* Get the song info table data. */
            songInfo = songInfoList[i];

            /* Extract info from the song info anchors. */
            songInfoAnchorList = songInfo.getElementsByTagName("a");
            for (j = 0; j < songInfoAnchorList.length; j++)
            {
                songInfoAnchor = songInfoAnchorList[j];
                switch (songInfoAnchor.className)
                {
                    case "tracklink" :
                        songInfo.trackID = songInfoAnchor.href.split("/").pop();
                        break;

                    case "listlink" :
                        if (songInfoAnchor.firstChild.data == "listen")
                            songInfo.listenNode = songInfoAnchor;
                        break;
                }
            }
        }

        /* Set up event handlers for and add     */
        /* song info to the song listen anchors. */
        for (i = 0; i < songInfoList.length; i++)
        {
            songInfo = songInfoList[i];

            if (songInfo.listenNode)
            {
                songInfo.listenNode.setAttribute("track_id", songInfo.trackID);
                songInfo.listenNode.addEventListener("click",
                                                     this.mHandlePlayClickFunc,
                                                     true);
            }
        }
    },


    /*
     * processPlaylistFile
     *
     *   --> playlistFile           Playlist file object to process.
     *
     *   This function processes the playlist file specified by playlistFile.
     */

    processPlaylistFile: function(playlistFile)
    {
        var                         playlist;

        /* Ensure the site library playlist exists. */
        this.mPlaylistManager.createPlaylist(this.siteDBTable + "_library",
                                             this.siteLibraryReadableName,
                                             "library",
                                             "library",
                                             this.mDBQuery);

        /* Reset the site playlist. */
        this.mPlaylistManager.deletePlaylist(this.siteDBTable, this.mDBQuery);
        playlist = this.mPlaylistManager.createPlaylist
                                                (this.siteDBTable,
                                                 this.sitePlaylistReadableName,
                                                 this.siteDBTable,
                                                 this.mPlaylistURL.spec,
                                                 this.mDBQuery);

        /* Process the playlist. */
        this.mPlaylistReaderManager.read("file:///" + playlistFile.path,
                                         this.siteDBGUID,
                                         this.siteDBTable,
                                         this.sitePlaylistMIMEType,
                                         "",
                                         false);

        /* Display the playlist viewer. */
        if (playlist.getNumEntries() > 0)
            onBrowserPlaylist();
    },


    /*
     * createTmpDir
     *
     *   This function creates a directory for temporary files used by the site.
     */

    createTmpDir: function()
    {
        var                         directoryService;
        var                         sysTmpDir;
        var                         siteTmpDir;
        var                         permissions;

        /* Get the directory service. */
        directoryService =
            Components.classes["@mozilla.org/file/directory_service;1"].
                            createInstance(Components.interfaces.nsIProperties);

        /* Get the system temporary directory and its permissions */
        /* and make a clone for the site temporary directory.     */
        sysTmpDir = directoryService.get("DefProfLRt", Components.interfaces.nsIFile);
        permissions = sysTmpDir.permissions;
        siteTmpDir = sysTmpDir.clone();

        /* Create an empty site directory. */
        siteTmpDir.append(this.siteDomain);
        if (siteTmpDir.exists())
            siteTmpDir.remove(true);
        siteTmpDir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE,
                          permissions);
        this.mSiteTmpDir = siteTmpDir;
    },


    /*
     * deleteTmpDir
     *
     *   This function deletes the directory used by the site for temporary
     * files.
     */

    deleteTmpDir: function()
    {
        if (this.mSiteTmpDir)
        {
            this.mSiteTmpDir.remove(true);
            this.mSiteTmpDir = null;
        }
    },


    /***************************************************************************
     *
     * Site search services.
     *
     **************************************************************************/

    /*
     * getSearchURL
     *
     *   --> searchTerm             Search term for which to get a URL.
     *
     *   <--                        Search URL.
     *
     *   This function returns a site search URL for the the search term
     * specified by searchTerm.
     */

    getSearchURL: function(searchTerm)
    {
        return ("http://hypem.com/search/" + searchTerm + "/1/");
    },


    /***************************************************************************
     *
     * Site event handlers.
     *
     **************************************************************************/

    /*
     * handlePlayClick
     *
     *   --> event                  Play click event.
     *
     *   This function handles the browser play media button click event
     * specified by event.
     *
     *   This function will initiate playback of the media specified by the
     * event.
     */

    handlePlayClick: function(event)
    {
        try { this.handlePlayClick1(event); }
        catch (err) { dump("handlePlayClick exception: " + err + "\n"); }
    },

    handlePlayClick1: function(event)
    {
        var                         listenNode;
        var                         trackID;
        var                         dbResult;
        var                         url = null;

        /* Get the play node info. */
        playNode = event.currentTarget;
        trackID = playNode.getAttribute("track_id");

        /* Get the track URL. */
        this.mDBQuery.resetQuery();
        this.mDBQuery.addQuery(  "SELECT url FROM library"
                               + " WHERE url LIKE \"%/" + trackID + "/%\"");
        this.mDBQuery.execute();
        this.mDBQuery.waitForCompletion();
        dbResult = this.mDBQuery.getResultObject();
        if (dbResult.getRowCount() > 0)
            url = dbResult.getRowCell(0, 0);

        /* Play the track and prevent further processing of the event. */
        if (url)
        {
            playExternalUrl(url, true);
            event.stopPropagation();
            event.preventDefault();
        }
    },


    /***************************************************************************
     *
     * Site nsIWebProgressListener interface.
     *
     **************************************************************************/

    /*
     * onStateChange
     *
     *   --> webProgress            Web progress object.
     *   --> request                Transfer request.
     *   --> stateFlags             New state flags.
     *   --> status                 Transfer status.
     *
     *   This function processes changes in the state of a web transfer.  The
     * progress of the transfer is specified by webProgress.  The request used
     * for the transfer is specified by request.  The new state flags are
     * specified by stateFlags, and the final status of the transfer is
     * specified by status.
     */

    onStateChange: function(webProgress, request, stateFlags, status)
    {
        /* Do nothing if the transfer is not complete. */
        if (!(  stateFlags
              & Components.interfaces.nsIWebProgressListener.STATE_STOP))
        {
            return;
        }

        /* Process the playlist file. */
        this.processPlaylistFile(this.mPlaylistFile);
    },


    onLocationChange: function()
    {
    },

    onProgressChange: function()
    {
    },

    onSecurityChange: function()
    {
    },

    onStatusChange: function()
    {
    },


    /***************************************************************************
     *
     * Site nsISupports interface.
     *
     **************************************************************************/

    /*
     * QueryInterface
     *
     *   --> interfaceID            Requested interface.
     *
     *   <-- NS_ERROR_NO_INTERFACE  Requested interface is not supported.
     *
     *   This function returns a playlist commands object implementing the
     * interface specified by interfaceID.
     */

    QueryInterface: function(interfaceID)
    {
        /* Check for supported interfaces. */
        if (   !interfaceID.equals(Components.interfaces.nsIWebProgressListener)
            && !interfaceID.equals(Components.interfaces.nsISupports)) 
        {
            throw (Components.results.NS_ERROR_NO_INTERFACE);
        }
    
        return (this);
    }
}


